#include "mainwindow.h"
#include "qevent.h"
#include <fstream>
#include <QInputDialog>
using namespace std;

class CustomTableWidgetItem : public QTableWidgetItem {
public:
    CustomTableWidgetItem(const QString &text) : QTableWidgetItem(text) {
        setTextAlignment(Qt::AlignCenter); // Вирівнюємо текст по центру
    }
    
    void setData(int role, const QVariant &value) override {
        QTableWidgetItem::setData(role, value);
    }
};

class CustomTableWidget : public QTableWidget {
    Q_OBJECT
#ifdef DEMO
    MainWindow *m_mainWindow;
#endif
public:
    CustomTableWidget(
#ifdef DEMO
        MainWindow *mainWindow,
#endif
        int rows, int columns) : QTableWidget(rows, columns)
#ifdef DEMO
        , m_mainWindow(mainWindow)
#endif
    {} 
signals:
    void cellClicked(QTableWidgetItem *item);
protected:
    void mousePressEvent(QMouseEvent *event) override {
        QTableWidgetItem *item = itemAt(event->pos());
        if (event->button() == Qt::RightButton) {
            // Ігнорування правої кнопки елементами таблички. Не має впливу на контекстне меню.
            event->ignore();
        }else
            emit cellClicked(item);
    }
    
    void mouseDoubleClickEvent(QMouseEvent *event) override {
        mousePressEvent(event);
    }
    
    void resizeEvent(QResizeEvent *event) override {
        QTableWidget::resizeEvent(event);
        QFont font;
        font.setPointSize(qMin(event->size().width()/columnCount(),
                               event->size().height()/rowCount()) / 3);
        for (int row = 0; row < rowCount(); ++row)for (int col = 0; col < columnCount(); ++col) {
                QTableWidgetItem *item = this->item(row, col);
                    item->setFont(font);
                    item->setTextAlignment(Qt::AlignCenter);
            }
    }
    void mouseMoveEvent(QMouseEvent *event) override {}
#ifdef DEMO
    void enterEvent(QEnterEvent *event) override{
        m_mainWindow->change(0,0,0);
    }
#endif
};
MainWindow::MainWindow() : splitter(new QSplitter(Qt::Horizontal))
{
    setCentralWidget(splitter);
    QWidget *secondWidget = new QWidget;
    QVBoxLayout *secondLayout = new QVBoxLayout(secondWidget);
    
    // Проміжок для відтягування вниз
    secondLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding));
    
    // Горизонтальне розташування для поля координат
    QHBoxLayout *coordLayout = new QHBoxLayout;
    coordLabelX = new QLabel("X:");
    coordLabelY = new QLabel("Y:");
    coordEditX = new QSpinBox;
    coordEditY = new QSpinBox;
    coordEditX->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    coordEditY->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    coordLayout->addWidget(coordLabelX);
    coordLayout->addWidget(coordEditX);
    coordLayout->addWidget(coordLabelY);
    coordLayout->addWidget(coordEditY);
    
    // Рядок для введення-виводу тексту
    QHBoxLayout *textLayout = new QHBoxLayout;
    valueLabel = new QLabel("Value:");
    valueEdit = new QSpinBox;
    valueEdit->setRange(0,1);
    valueEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed); // Змінено на QSizePolicy::Expanding
    textLayout->addWidget(valueLabel);
    textLayout->addWidget(valueEdit);
    secondLayout->addLayout(coordLayout);
    secondLayout->addLayout(textLayout);
    
    // Горизонтальне розташування для кнопок "Set" і "Get"
    QHBoxLayout *buttonsLayout = new QHBoxLayout;
    buttonsLayout->setAlignment(Qt::AlignHCenter); // Вирівняти по центру
    
    setButton = new QPushButton("Set");
    getButton = new QPushButton("Get");
    connect(setButton, &QPushButton::clicked, this, &MainWindow::setElement);
    connect(getButton, &QPushButton::clicked, this, &MainWindow::getElement);
    buttonsLayout->addWidget(setButton);
    buttonsLayout->addSpacing(20);
    
    buttonsLayout->addWidget(getButton);
    secondLayout->addLayout(buttonsLayout);
    
    QLabel *secondLabel = new QLabel("Enter value for n:");
    secondLayout->addWidget(secondLabel, 0, Qt::AlignHCenter | Qt::AlignBottom);
    spinBox = new QSpinBox;
    spinBox->setRange(2, 20);
    spinBox->setValue(6);
    spinBox->selectAll();
    secondLayout->addWidget(spinBox);
    submitButton = new QPushButton("Submit");
    secondLayout->addWidget(submitButton);
    connect(submitButton, &QPushButton::clicked, this, &MainWindow::handleSubmit);
    
    splitter->addWidget(secondWidget);
    
    splitter->setSizes({2,1});
    
    newAct = new QAction(tr("&New"), this);
    newAct->setShortcuts(QKeySequence::New);
    newAct->setStatusTip(tr("Create a new file"));
    connect(newAct, &QAction::triggered, this, &MainWindow::newFile);
    
    openAct = new QAction(tr("&Open..."), this);
    openAct->setShortcuts(QKeySequence::Open);
    openAct->setStatusTip(tr("Open an existing file"));
    connect(openAct, &QAction::triggered, this, &MainWindow::open);
    
    saveAct = new QAction(tr("&Save..."), this);
    saveAct->setShortcuts(QKeySequence::Save);
    saveAct->setStatusTip(tr("Save the document to disk"));
    connect(saveAct, &QAction::triggered, this, &MainWindow::save);
    
    exitAct = new QAction(tr("E&xit"), this);
    exitAct->setShortcuts(QKeySequence::Quit);
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, &QAction::triggered, this, &QWidget::close);
    
    aboutAct = new QAction(tr("&About"), this);
    aboutAct->setShortcuts(QKeySequence::HelpContents);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, &QAction::triggered, this, &MainWindow::about);
    
    aboutQtAct = new QAction(tr("About &Qt"), this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, &QAction::triggered, qApp, &QApplication::aboutQt);
    
    fillAct = new QAction(tr("&Fill"), this);
    fillAct->setStatusTip(tr("Fill with cute figure"));
    connect(fillAct, &QAction::triggered, this, &MainWindow::fill); // замінено з aboutAct на fillAct
    
    getAct = new QAction(tr("&Get value"), this);
    getAct->setShortcuts(QKeySequence::Find);
    getAct->setStatusTip(tr("Get value of element num"));
    connect(getAct, &QAction::triggered, this, &MainWindow::openGetValueDialog);
    
    setAct = new QAction(tr("&Set value"), this);
    setAct->setShortcut(QKeySequence::Paste);
    setAct->setStatusTip(tr("Set value of element num"));
    connect(setAct, &QAction::triggered, this, &MainWindow::openSetValueDialog);
    
    clearAct = new QAction(tr("&Clear"), this);
    clearAct->setShortcuts(QKeySequence::Delete);
    clearAct->setStatusTip(tr("Set all values to 0"));
    connect(clearAct, &QAction::triggered, this, &MainWindow::clear);
    
    saveAct->setEnabled(false);
    
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(newAct);
    fileMenu->addAction(saveAct);
    fileMenu->addAction(openAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);
    tableMenu = menuBar()->addMenu(tr("&Table"));
    tableMenu->setEnabled(false);
    tableMenu->addAction(fillAct);
    tableMenu->addAction(getAct);
    tableMenu->addAction(setAct);
    tableMenu->addAction(clearAct);
    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);
    
    statusBar()->showMessage(tr("Very unstable debug alpha"));
    
    setWindowTitle(tr("oop_lab6_prog"));
    setMinimumSize(320, 320);
    resize(960, 540);
    coordEditX->setEnabled(false);
    coordEditY->setEnabled(false);
    valueEdit->setEnabled(false);
    valueLabel->setEnabled(false);
    coordLabelY->setEnabled(false);
    coordLabelX->setEnabled(false);
    getButton->setEnabled(false);
    setButton->setEnabled(false);
}

void MainWindow::change(QTableWidgetItem* item, char val) {
    bool newval;
    if (val==50||val==2)
        newval=!(item->text().toShort());
    else
        newval=(val==1||val==49);
    item->setText(newval ? "1" : "0");
    QVariantAnimation *colorAnimation = new QVariantAnimation();
    colorAnimation->setDuration(400);
    colorAnimation->setStartValue(valueEdit->value() ? QColor(QColorConstants::Svg::lightgreen) : QColor(QColorConstants::Svg::indianred));
    colorAnimation->setEndValue(newval ? QColor(Qt::green) : QColor(Qt::red));
    connect(colorAnimation, &QVariantAnimation::valueChanged, [=](const QVariant &value) {
        item->setBackground(QBrush(value.value<QColor>()));
    });

    colorAnimation->start();
}
void MainWindow::change(char x,char y, char val) {change(tableWidget->item(x,y),val);}
void MainWindow::setElement() {
    change(coordEditY->value()-1, coordEditX->value()-1,QString::number(valueEdit->value()).at(0).toLatin1());
}

void MainWindow::getElement() {
        QTableWidgetItem *item = tableWidget->item(coordEditY->value()-1, coordEditX->value()-1);
            valueEdit->setValue(item->text().toInt());
}
void MainWindow::createTable(int n)
{
    // Створення нового вмісту справа
    QWidget *secondWidget = new QWidget;
    QVBoxLayout *secondLayout = new QVBoxLayout(secondWidget);
    
    tableWidget = new CustomTableWidget(
#ifdef DEMO
        this,
#endif
        n, n);
    for (int row = 0; row < n; ++row) {
        for (int col = 0; col < n; ++col) {
            CustomTableWidgetItem *item = new CustomTableWidgetItem("0");
            item->setData(Qt::BackgroundRole, QColor(Qt::red));
            tableWidget->setItem(row, col, item); // Використовуємо tableWidget
        }
    }
    tableWidget->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    tableWidget->verticalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    secondLayout->addWidget(tableWidget);
    
    // Підключення сигналу до слоту після ініціалізації tableWidget
    connect(tableWidget, &CustomTableWidget::cellClicked, this, [this](QTableWidgetItem *item) {
        change(item, '2');
    });
    
    if (splitter->count() > 1)
        delete splitter->widget(0);
    
    splitter->insertWidget(0, secondWidget);
    saveAct->setEnabled(true);
    tableMenu->setEnabled(true);
    coordEditX->setRange(1, size);
    coordEditY->setRange(1, size);
    coordEditX->setEnabled(true);
    coordEditY->setEnabled(true);
    valueEdit->setEnabled(true);
    valueLabel->setEnabled(true);
    coordLabelY->setEnabled(true);
    coordLabelX->setEnabled(true);
    getButton->setEnabled(true);
    setButton->setEnabled(true);
}

#ifndef QT_NO_CONTEXTMENU
void MainWindow::contextMenuEvent(QContextMenuEvent *event)
{
    QMenu menu(this);
    
    menu.addAction(fillAct);
    menu.addAction(clearAct);
    menu.addAction(saveAct);
    menu.addAction(aboutAct);
    menu.exec(event->globalPos());
}
#endif // QT_NO_CONTEXTMENU

void MainWindow::newFile()
{
    submitButton->setText("Submit"); // Встановлення тексту кнопки назад на "Submit"
    if (splitter->count() > 1) {
        QWidget *existingWidget = splitter->widget(0);
        if (existingWidget) {
            delete existingWidget;
        }
    }
    saveAct->setEnabled(false);
    tableMenu->setEnabled(false);
    coordEditX->setEnabled(false);
    coordEditY->setEnabled(false);
    valueEdit->setEnabled(false);
    valueLabel->setEnabled(false);
    coordLabelY->setEnabled(false);
    coordLabelX->setEnabled(false);
    getButton->setEnabled(false);
    setButton->setEnabled(false);
    valueEdit->setValue(1);
}

bool MainWindow::open()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"), "", tr("Text Files (*.txt)"));
    if (fileName.isEmpty())return 1;
    
    std::ifstream file(fileName.toStdString(), std::ios::binary);
    if (!file.is_open()) {
        QMessageBox::warning(this, tr("Error"), tr("Could not open file for reading"));
        return 1;
    }
    
    char size_char;
    file >> size_char;
    size = size_char - '0';
    if (!(splitter->count()-1)) createTable(size);
    
    for (int row = 0; row < size; ++row) {
        for (int col = 0; col < size; ++col) {
            char value_char;
            file >> value_char;
            bool value = value_char-48;
            QTableWidgetItem *item = tableWidget->item(row, col);
            if (item) {
                if (value) {
                    item->setText("1");
                    item->setData(Qt::BackgroundRole, QColor(Qt::green));
                } else {
                    item->setText("0");
                    item->setData(Qt::BackgroundRole, QColor(Qt::red));
                }
            }
        }
    }
    valueEdit->setValue(1);
    file.close();return 1;
}

void MainWindow::save()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), "", tr("Text Files (*.txt)"));
    if (fileName.isEmpty())return;
    
    std::ofstream file(fileName.toStdString(), std::ios::binary);
    if (!file.is_open()) {
        QMessageBox::warning(this, tr("Error"), tr("Could not open file for writing"));
        return;
    }
    file<<char(size+48)<<std::endl;
    // Цикл для запису значень комірок таблиці
    for (int row = 0; row < size; ++row) {
        for (int col = 0; col < size; ++col) {
            QTableWidgetItem *item = tableWidget->item(row, col);
            if (item) {
                char value = (item->text() == "1") ? 1 : 0; // Перевіряємо текст комірки і перетворюємо на відповідне значення
                file<<char(value+48); // Записуємо значення у файл
            }
        }file<<std::endl;
    }
    
    file.close();
};


class SetValueDialog : public QDialog {
public:
    SetValueDialog(MainWindow *mainWindow, CustomTableWidget *tableWidget, QWidget *parent = nullptr, int size = 1) : QDialog(parent), m_mainWindow(mainWindow), m_tableWidget(tableWidget) {
        QLabel *label = new QLabel("Введіть координати:");
        QSpinBox *spinBoxX = new QSpinBox();
        QSpinBox *spinBoxY = new QSpinBox();
        QPushButton *okButton = new QPushButton("OK");
        QPushButton *cancelButton = new QPushButton("Відміна");
        
        // Налаштування діапазонів для спінбоксів
        spinBoxX->setRange(1, size);
        spinBoxY->setRange(1, size);
        
        // Налаштування розміщення віджетів у вікні
        QVBoxLayout *layout = new QVBoxLayout();
        layout->addWidget(label);
        
        QHBoxLayout *spinBoxLayout = new QHBoxLayout();
        spinBoxLayout->addWidget(spinBoxX);
        spinBoxLayout->addWidget(spinBoxY);
        layout->addLayout(spinBoxLayout);
        
        QHBoxLayout *buttonLayout = new QHBoxLayout();
        buttonLayout->addWidget(okButton);
        buttonLayout->addWidget(cancelButton);
        layout->addLayout(buttonLayout);
        
        setLayout(layout);
        
        connect(okButton, &QPushButton::clicked, this, [this, spinBoxX, spinBoxY]() {
            m_mainWindow->change(spinBoxX->value()-1, spinBoxY->value()-1, QInputDialog::getInt(this, tr("Введіть нове значення"), tr("Нове значення:"), 1, 0, 1));
            close();
        });
        
        connect(cancelButton, &QPushButton::clicked, this, &QDialog::close);
    }
    
private:
    CustomTableWidget *m_tableWidget;
    MainWindow *m_mainWindow;
};
class GetValueDialog : public QDialog {
public:
    GetValueDialog(CustomTableWidget *tableWidget, QWidget *parent = nullptr, int size = 1) : QDialog(parent), m_tableWidget(tableWidget) {
        QLabel *label = new QLabel("Введіть координнати:");
        QSpinBox *spinBoxX = new QSpinBox();
        QSpinBox *spinBoxY = new QSpinBox();
        QPushButton *okButton = new QPushButton("OK");
        QPushButton *cancelButton = new QPushButton("Відміна");
        
        // Налаштування діапазонів для спінбоксів
        spinBoxX->setRange(1, size);
        spinBoxY->setRange(1, size);
        
        // Налаштування розміщення віджетів у вікні
        QVBoxLayout *layout = new QVBoxLayout();
        layout->addWidget(label);
        
        QHBoxLayout *spinBoxLayout = new QHBoxLayout();
        spinBoxLayout->addWidget(spinBoxX);
        spinBoxLayout->addWidget(spinBoxY);
        layout->addLayout(spinBoxLayout);
        
        QHBoxLayout *buttonLayout = new QHBoxLayout();
        buttonLayout->addWidget(okButton);
        buttonLayout->addWidget(cancelButton);
        layout->addLayout(buttonLayout);
        
        setLayout(layout);
        
        // З'єднання сигналів та слотів
        connect(okButton, &QPushButton::clicked, this, [this, spinBoxX, spinBoxY]() {
            // Отримання введених координат
            int x = spinBoxX->value();
            int y = spinBoxY->value();
            
            // Отримання значення з комірки таблиці за введеними координатами
            QTableWidgetItem *item = m_tableWidget->item(y - 1, x - 1);
            if (item) {
                QString value = item->text();
                
                // Створення та відображення вікна зі значенням
                QMessageBox::information(this, tr("Значення"), tr("Значення за координатами (%1, %2): %3").arg(x).arg(y).arg(value));
            }
            
            close();
        });
        
        connect(cancelButton, &QPushButton::clicked, this, &QDialog::close);
    }
    
private:
    CustomTableWidget *m_tableWidget; // вказівник на таблицю
};

void MainWindow::openGetValueDialog()
{
    class GetValueDialog getValueDialog(tableWidget, this, size);
    getValueDialog.exec();
}

void MainWindow::openSetValueDialog()
{
    class SetValueDialog setValueDialog(this,tableWidget, this, size);
    setValueDialog.exec();
}
void MainWindow::fill()
{
    if (!(splitter->count()-1)) return;
    
    for (int i = 0; i < size; ++i) {
        for (int j = 0; j < size; ++j) {
            // int verticalDistance = qMin(i, size - i - 1);
            // int horizontalDistance = qMin(j, size - j - 1);
            //bool cond
            // cond=horizontalDistance <= verticalDistance;
            // cond=i>=size/2&&j>=size/2||i<size/2&&j<size/2
            if (i<j) {
                QTableWidgetItem *item = tableWidget->item(j,i);
                if (item) {
                    item->setText("1");
                    item->setData(Qt::BackgroundRole, QColor(Qt::green));
                }
            } else {
                QTableWidgetItem *item = tableWidget->item(j,i);
                if (item) {
                    item->setText("0");
                    item->setData(Qt::BackgroundRole, QColor(Qt::red));
                }
            }
        }
    }
}


void MainWindow::about()
{
    QMessageBox::about(this, tr("About oop_lab6_prog"),
            tr("The <b>oop_lab6_prog</b> is a horrible example of laboratory work.<br>Created by Ivan."));
}
void MainWindow::clear()
{
    for (char i=0;i<size*size;i++){
        QTableWidgetItem *item = tableWidget->item(i%size,i/size);
        if (item) {
            item->setText("0");
            item->setData(Qt::BackgroundRole, QColor(Qt::red));
        }
    }
}

void MainWindow::handleSubmit() {
    submitButton->setText("Refill");
    size=spinBox->value();
    createTable(size);
}
#include "mainwindow.moc"
